@@ -61,6 +61,7 @@ static FILE *import_file = NULL;
 static int ckey = 0;
 static int skey;
 static unsigned char *macro;
+static bool macro_is_name;
 static int mouse_key;
 static unsigned short active_keycode = -1;
 static int just_pressed_shift = 0;
@@ -264,7 +265,7 @@ static void low_battery_checker(CFRunLoopTimerRef timer, void *info) {
         [mainWindow setFrameOrigin:pt];
     }
     
-    sz.width = 301;
+    sz.width = 373;
     sz.height = state.printWindowKnown ? state.printWindowHeight : 600;
     [printWindow setContentSize:sz];
     [printView initialUpdate];
@@ -463,7 +464,7 @@ static void low_battery_checker(CFRunLoopTimerRef timer, void *info) {
 - (IBAction) clearPrintOut:(id)sender {
     printout_top = printout_bottom = 0;
     NSSize s;
-    s.width = 286;
+    s.width = 358;
     s.height = 0;
     [printView setFrameSize:s];
 }
@@ -607,7 +608,7 @@ static char version[32] = "";
 }
 
 - (void) mouseDown3 {
-    macro = skin_find_macro(ckey);
+    macro = skin_find_macro(ckey, &macro_is_name);
     shell_keydown();
     mouse_key = 1;
 }
@@ -773,34 +774,40 @@ static void shell_keydown() {
     // EventAvail is an annoying omission of the iPhone API.)
     
     if (macro != NULL) {
-        if (*macro == 0) {
-            squeak();
-            return;
-        }
-        bool one_key_macro = macro[1] == 0 || (macro[2] == 0 && macro[0] == 28);
-        if (!one_key_macro) {
-            skin_display_set_enabled(false);
-        }
-        while (*macro != 0) {
+        if (macro_is_name) {
             we_want_cpu = 1;
-            keep_running = core_keydown(*macro++, &enqueued, &repeat);
+            keep_running = core_keydown_command((const char *) macro, &enqueued, &repeat);
             we_want_cpu = 0;
-            if (*macro != 0 && !enqueued)
-                core_keyup();
-        }
-        if (!one_key_macro) {
-            skin_display_set_enabled(true);
-            skin_repaint_display();
-            /*
-             skin_repaint_annunciator(1, ann_updown);
-             skin_repaint_annunciator(2, ann_shift);
-             skin_repaint_annunciator(3, ann_print);
-             skin_repaint_annunciator(4, ann_run);
-             skin_repaint_annunciator(5, ann_battery);
-             skin_repaint_annunciator(6, ann_g);
-             skin_repaint_annunciator(7, ann_rad);
-             */
-            repeat = 0;
+        } else {
+            if (*macro == 0) {
+                squeak();
+                return;
+            }
+            bool one_key_macro = macro[1] == 0 || (macro[2] == 0 && macro[0] == 28);
+            if (!one_key_macro) {
+                skin_display_set_enabled(false);
+            }
+            while (*macro != 0) {
+                we_want_cpu = 1;
+                keep_running = core_keydown(*macro++, &enqueued, &repeat);
+                we_want_cpu = 0;
+                if (*macro != 0 && !enqueued)
+                    core_keyup();
+            }
+            if (!one_key_macro) {
+                skin_display_set_enabled(true);
+                skin_repaint_display();
+                /*
+                skin_repaint_annunciator(1, ann_updown);
+                skin_repaint_annunciator(2, ann_shift);
+                skin_repaint_annunciator(3, ann_print);
+                skin_repaint_annunciator(4, ann_run);
+                skin_repaint_annunciator(5, ann_battery);
+                skin_repaint_annunciator(6, ann_g);
+                skin_repaint_annunciator(7, ann_rad);
+                */
+                repeat = 0;
+            }
         }
     } else {
         we_want_cpu = 1;
@@ -896,7 +903,7 @@ void calc_keydown(NSString *characters, NSUInteger flags, unsigned short keycode
                     key_macro = entry->macro;
                     break;
                 } else {
-                    if (key_macro == NULL)
+                    if (cshift && key_macro == NULL)
                         key_macro = entry->macro;
                 }
             }
@@ -966,7 +973,7 @@ void calc_keydown(NSString *characters, NSUInteger flags, unsigned short keycode
                 if (c <= 37)
                     macrobuf[p++] = c;
                 else {
-                    unsigned char *m = skin_find_macro(c);
+                    unsigned char *m = skin_find_macro(c, &macro_is_name);
                     if (m != NULL)
                         while (*m != 0 && p < 1023)
                             macrobuf[p++] = *m++;
@@ -974,8 +981,10 @@ void calc_keydown(NSString *characters, NSUInteger flags, unsigned short keycode
             }
             macrobuf[p] = 0;
             macro = macrobuf;
-        } else
+        } else {
             macro = key_macro;
+            macro_is_name = false;
+        }
         shell_keydown();
         mouse_key = 0;
         active_keycode = keycode;
@@ -1173,7 +1182,10 @@ void shell_print(const char *text, int length,
                 fwrite("\357\273\277", 1, 3, print_txt);
         }
         
-        shell_spool_txt(text, length, txt_writer, txt_newliner);
+        if (text != NULL)
+            shell_spool_txt(text, length, txt_writer, txt_newliner);
+        else
+            shell_spool_bitmap_to_txt(bits, bytesperline, x, y, width, height, txt_writer, txt_newliner);
     done_print_txt:;
     }
     
